import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import { useState, useRef, useEffect } from 'react';
import dayjs from 'dayjs';
import duration from 'dayjs/plugin/duration';
import { isDef } from '../_util/index';
dayjs.extend(duration);
var formatStringCache = {};
;
var limitUnit = ['s', 'm', 'H', 'D', 'M', 'Y'];
var limitUnitMap = {};
limitUnit.forEach(function (item, idx) {
  limitUnitMap[item] = idx;
});
export function getTimeObj(duration, endTime, parseFormatString) {
  var timeObj = dayjs.duration(duration < 0 ? 0 : duration, 'seconds');

  if (isDef(endTime) && isDef(parseFormatString)) {
    var thisMoment = dayjs();
    var targetMoment = dayjs(endTime, parseFormatString);

    if (targetMoment.isAfter(thisMoment)) {
      timeObj = dayjs.duration(targetMoment.diff(thisMoment));
    }
  }

  return timeObj;
}
export function parseFormatStr(formatString) {
  if (formatStringCache.hasOwnProperty(formatString)) {
    return formatStringCache[formatString];
  }

  var strArr = [];
  var piece = '';

  for (var i = 0; i < formatString.length;) {
    var c = formatString[i];

    if (limitUnitMap.hasOwnProperty(c)) {
      var j = i + 1;

      if (piece) {
        strArr.push(piece);
        piece = '';
      }

      while (j < formatString.length) {
        if (formatString[j] === c) {
          j++;
        } else {
          break;
        }
      }

      strArr.push(formatString.slice(i, j));
      i = j;
    } else {
      piece += c;
      i++;
    }
  }

  if (piece) {
    strArr.push(piece);
  }

  formatStringCache[formatString] = strArr;
  return strArr;
}
export function getTimeStr(timeObj, formatString) {
  var strArr = parseFormatStr(formatString);
  var unitSet = new Set();

  for (var i = 0; i < strArr.length; i++) {
    var c = strArr[i][0];

    if (limitUnitMap.hasOwnProperty(c)) {
      unitSet.add(c);
    }
  }

  var unitArr = _toConsumableArray(unitSet).sort(function (a, b) {
    return limitUnitMap[b] - limitUnitMap[a];
  });

  var copyedTimeObj = timeObj.clone();
  var m = {};

  for (var _i = 0; _i < unitArr.length; _i++) {
    var mathFn = _i === unitArr.length - 1 ? Math.round : Math.floor;

    switch (unitArr[_i]) {
      case 'Y':
        m.Y = mathFn(copyedTimeObj.asYears());
        copyedTimeObj = copyedTimeObj.subtract(m.Y, 'years');
        break;

      case 'M':
        m.M = mathFn(copyedTimeObj.asMonths());
        copyedTimeObj = copyedTimeObj.subtract(m.M, 'months');
        break;

      case 'D':
        m.D = mathFn(copyedTimeObj.asDays());
        copyedTimeObj = copyedTimeObj.subtract(m.D, 'days');
        break;

      case 'H':
        m.H = mathFn(copyedTimeObj.asHours());
        copyedTimeObj = copyedTimeObj.subtract(m.H, 'hours');
        break;

      case 'm':
        m.m = mathFn(copyedTimeObj.asMinutes());
        copyedTimeObj = copyedTimeObj.subtract(m.m, 'minutes');
        break;

      case 's':
        m.s = mathFn(copyedTimeObj.asSeconds());
        copyedTimeObj = copyedTimeObj.subtract(m.s, 'seconds');
        break;
    }
  }

  return strArr.map(function (item) {
    var c = item[0];

    if (limitUnitMap.hasOwnProperty(c)) {
      return "".concat(m[c]).padStart(item.length, '0');
    }

    return item;
  }).join('');
}
export default (function (props) {
  var _props$formatString = props.formatString,
      formatString = _props$formatString === void 0 ? 'mm:ss' : _props$formatString,
      _props$duration = props.duration,
      duration = _props$duration === void 0 ? 0 : _props$duration,
      endTime = props.endTime,
      _props$parseFormatStr = props.parseFormatString,
      parseFormatString = _props$parseFormatStr === void 0 ? 'YYYY-MM-DD HH:mm:ss' : _props$parseFormatStr,
      onFinished = props.onFinished,
      children = props.children;
  var timeObjRef = useRef(getTimeObj(duration, endTime, parseFormatString));

  var _useState = useState(getTimeStr(timeObjRef.current, formatString)),
      _useState2 = _slicedToArray(_useState, 2),
      timeStr = _useState2[0],
      setTimeStr = _useState2[1];

  useEffect(function () {
    if (timeObjRef.current.asSeconds() < 1) {
      typeof onFinished === 'function' && onFinished();
    } else {
      var timer = setInterval(function () {
        if (timeObjRef.current.asSeconds() < 1) {
          clearInterval(timer);
          typeof onFinished === 'function' && onFinished();
        } else {
          timeObjRef.current = timeObjRef.current.subtract(1, 'seconds');
          setTimeStr(getTimeStr(timeObjRef.current, formatString));
        }
      }, 1000);
      return function () {
        clearInterval(timer);
      };
    }
  }, []);
  return typeof children === 'function' ? children(timeStr) : null;
});