## PageBackListener 页面后退监听器

工具组件，请在顶层页面级别的组件中使用。

该工具接收一个回调函数，当用户在当前页面触发后退操作时，先执行该函数。

#### 注意点

+ 请不要将此监听当作一个必然的保证，并不能绝对的拦截/屏蔽用户的后退操作

+ 点击浏览器的关闭所有页面按钮将不会触发该操作

+ 请在移动端体验该功能

#### 实现原理

使用 history.pushState 创建 history，使后退操作返回了上一个 history。

#### 使用场景

+ 离开页面前弹窗挽留

+ 仿生app离开模式，离开页面需要点击2次后退按钮（第一次toast提示）

#### 基本使用

```javascript
import {
  PageBackListener,
} from 'ui-hub';

function Demo(props) {
  const listenerRef = React.createRef();
  useEffect(() => {
    listenerRef.current = new PageBackListener(() => {
      // do something you want
      // like Modal.confirm()
    });
    return () => {
      listenerRef.current.removeEffect();
    }
  }, []);
  return (
    <div>
      <button
        onClick={() => {
          listenerRef.current && listenerRef.current.removeListener();
        }}
      >
        移除后退监听逻辑
      </button>
    </div>
  )
}
```

###### 核心代码

```javascript
// 实例化
const l = new PageBackListener(callbackFn);
// 移除副作用
l.removeEffect();
// 移除整个事件
l.removeListener();
```

#### 重新监听

`callbackFn` 的类型为 `(reListen?: () => void) => void`, 你可以使用 `callbackFn` 中提供的第一个参数来触发一次重新监听

```javascript
const l = new PageBackListener((reListener) => {
  setTimeout(() => {
    reListener();
  }, 2000);
});
```

###### 应用-模拟客户端点两次后退键退出APP

```javascript
import {
  PageBackListener,
  Toast,
} from 'ui-hub';

function Demo(props) {
  useEffect(() => {
    const l = new PageBackListener((reListen) => {
      // Toast组件默认展示2s
      Toast.info('再次点击返回键退出APP');
      // 如果2s内用户不退出 再次触发一次监听
      setTimeout(() => {
        reListen();
      }, 2000);
    });
    return () => {
      l.removeEffect();
    }
  }, []);
  return (
    <div />
  );
}
```

#### API

+ removeEffect

清除副作用（但是不会清除返回逻辑），该方法一般应用在 useEffect 的清除返回回调中或 componentWillUnmount 生命周期中。

+ removeListener

移除整个返回监听逻辑