"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireWildcard(require("react"));

var _classnames = _interopRequireDefault(require("classnames"));

var _CloseCircleFilled = _interopRequireDefault(require("@ant-design/icons/CloseCircleFilled"));

var _index = require("../_util/index");

var _index2 = _interopRequireDefault(require("../_hook/useUpdateEffect/index"));

var _dict = require("../_config/dict");

;

function InputItem(props, ref) {
  var defaultValue = props.defaultValue,
      onChange = props.onChange,
      _props$maxLength = props.maxLength,
      maxLength = _props$maxLength === void 0 ? Infinity : _props$maxLength,
      _props$prefixCls = props.prefixCls,
      prefixCls = _props$prefixCls === void 0 ? _dict.defaultPrefixCls : _props$prefixCls,
      placeholder = props.placeholder,
      className = props.className,
      clear = props.clear,
      type = props.type,
      addonAfter = props.addonAfter,
      addonBefore = props.addonBefore,
      disabled = props.disabled,
      onHandleSelectionPos = props.onHandleSelectionPos,
      restProps = (0, _objectWithoutProperties2["default"])(props, ["defaultValue", "onChange", "maxLength", "prefixCls", "placeholder", "className", "clear", "type", "addonAfter", "addonBefore", "disabled", "onHandleSelectionPos"]);

  var _useState = (0, _react.useState)(props.value || defaultValue || ''),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      value = _useState2[0],
      setValue = _useState2[1];

  var inputRef = (0, _react.useRef)(null);
  var selectionRef = (0, _react.useRef)(null);
  (0, _index2["default"])(function () {
    if (props.value !== undefined && props.value !== value) {
      setValue(props.value || '');
    }
  }, [props.value, value]);

  var handlePosition = function handlePosition() {
    if (selectionRef.current) {
      try {
        inputRef.current.selectionStart = selectionRef.current.start || null;
        inputRef.current.selectionEnd = selectionRef.current.end || null;
      } catch (e) {}
    }
  };

  var focus = function focus() {
    inputRef.current.focus();
  };

  var blur = function blur() {
    inputRef.current.blur();
  };

  var select = function select() {
    inputRef.current.select();
  };

  (0, _react.useEffect)(function () {
    handlePosition();
  }, [value]);
  (0, _react.useImperativeHandle)(ref, function () {
    return {
      handlePosition: handlePosition,
      focus: focus,
      blur: blur,
      select: select
    };
  });

  var onRealChangeValue = function onRealChangeValue(v) {
    if (disabled) {
      return;
    }

    if (props.value === undefined) {
      setValue(v);
    }

    if (typeof onChange === 'function') {
      onChange(v);
    }
  };

  var onChangeValue = function onChangeValue(e) {
    var el = e.target;
    var elValue = el.value; // 如果当前值已经达到最大值,继续输入超过最大值的值,return掉

    if (value.length >= maxLength && elValue.length > maxLength) {
      return;
    }

    var currentValue = elValue;

    if (maxLength !== Infinity) {
      currentValue = currentValue.slice(0, maxLength);
    }

    try {
      var inputSelection = {};
      inputSelection.start = el.selectionStart;
      inputSelection.end = el.selectionEnd;

      if (inputSelection.start && inputSelection.start > maxLength + 1) {
        inputSelection.start = maxLength + 1;
      }

      if (inputSelection.end && inputSelection.end > maxLength + 1) {
        inputSelection.end = maxLength + 1;
      }

      if (typeof onHandleSelectionPos === 'function') {
        var _onHandleSelectionPos = onHandleSelectionPos(currentValue, inputSelection.start, inputSelection.end),
            start = _onHandleSelectionPos.start,
            end = _onHandleSelectionPos.end;

        if ((0, _index.isDef)(start) && (0, _index.isDef)(end)) {
          inputSelection.start = start;
          inputSelection.end = end;
        }
      }

      selectionRef.current = inputSelection;
    } catch (e) {}

    if (elValue.length > maxLength) {
      setTimeout(function () {
        handlePosition();
      });
    }

    if (currentValue !== value) {
      onRealChangeValue(currentValue);
    }
  };

  var onClearValue = function onClearValue() {
    onRealChangeValue('');
  };

  var needRenderClearIcon = clear && !disabled && value;
  var hasAddOnBefore = !!addonBefore;
  var hasAddOnAfter = needRenderClearIcon || !!addonAfter;
  var inputType = 'text';
  var patternProps;

  if (type === 'phone') {
    inputType = 'tel';
  } else if (type === 'digit') {
    inputType = 'number';
    patternProps = {
      pattern: '[0-9]*'
    };
  } else if (type) {
    inputType = type;
  }

  return /*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])("".concat(prefixCls, "-input"), className)
  }, hasAddOnBefore && /*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])("".concat(prefixCls, "-input-addons"))
  }, addonBefore), /*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])("".concat(prefixCls, "-input-container"))
  }, /*#__PURE__*/_react["default"].createElement("input", (0, _extends2["default"])({}, patternProps, restProps, {
    ref: inputRef,
    value: value,
    type: inputType,
    onChange: onChangeValue,
    placeholder: placeholder,
    disabled: disabled,
    readOnly: disabled
  }))), hasAddOnAfter && /*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])("".concat(prefixCls, "-input-addons"))
  }, needRenderClearIcon && /*#__PURE__*/_react["default"].createElement(_CloseCircleFilled["default"], {
    className: "".concat(prefixCls, "-input-icon"),
    onClick: onClearValue
  }), addonAfter));
}

;

var _default = /*#__PURE__*/(0, _react.forwardRef)(InputItem);

exports["default"] = _default;